<?php namespace App\Commands;

use CodeIgniter\CLI\BaseCommand;
use CodeIgniter\CLI\CLI;

class MakeController extends BaseCommand
{
	protected $group = 'ErtikazOS';
	protected $name = 'make:controller';
	protected $description = 'Create a new controller class.';
	protected $usage = 'make:controller [namespace] [controller] [model]';
	protected $arguments = [
		'namespace' => 'The namespace of your application.',
		'controller' => 'The name of the controller class.',
		'model' => 'The name of the model class.',
	];

	public function run(array $params = [])
	{
		if (!isset($params[0]) || empty($params[0]))
		{
			CLI::error('"namespace" argument is missing.');
			CLI::write('');
			return;
		}
		if (!isset($params[1]) || empty($params[1]))
		{
			CLI::error('"controller" argument is missing.');
			CLI::write('');
			return;
		}
		if (!isset($params[2]) || empty($params[2]))
		{
			CLI::error('"model" argument is missing.');
			CLI::write('');
			return;
		}

		// Class names must start with an uppercase letter.
		$namespace  = ucfirst(array_shift($params));
		$classname  = ucfirst(array_shift($params));
		$modelclass = ucfirst(array_shift($params));
		// model variable name 
		$modelname  = strtolower($modelclass);
		// used in Routes.php
		$classroute = str_replace('\\', '/', $namespace.'/'.$classname);
		$classroute = strtolower(trim($classroute, '/'));

		//--------------------------------------------------------------------
		// Controller template
		//--------------------------------------------------------------------
		$template = <<<EOT
<?php namespace App\Controllers\{namespace};

use App\Models\{namespace}\{modelclass};
use App\Entities\{namespace}\{modelclass}Entity;
use App\Controllers\BaseController;

class {classname} extends BaseController
{
	protected \${modelname};
	function __construct()
	{
		\$this->{modelname} = new {modelclass}();
	}

	public function index()
	{
		\$limit  = 10;
		\$offset = 0;
		\${modelname} = \$this->{modelname};
		\$rows = \${modelname}->paginate(\$limit, '{modelname}');
		return view('{classroute}/index', ['rows'=>\$rows, 'pager'=>\${modelname}->pager]);
	}

	public function show(\$id = null)
	{
		\$row = \$this->{modelname}->find(\$id);
		if(!\$row)
		{
			return redirect()
				->to('/{classroute}')
				->with('errors', 'Data is not found.');
		}
		return view('{classroute}/show', ['row'=>\$row]);
	}

	public function create()
	{
		return view('{classroute}/create');
	}

	public function postCreate()
	{
		\${modelname}Entity = new {modelclass}Entity();
		\${modelname}Entity->fill(\$this->request->getPost());
		if(!\$this->{modelname}->save(\${modelname}Entity))
		{
			return redirect()
				->to('/{classroute}/create')
				->withInput()
				->with('errors', \$this->{modelname}->errors());
		}
		return redirect()
			->to('/{classroute}')
			->with('success', 'Data has been saved.');
	}

	public function update(\$id = null)
	{
		\$row = \$this->{modelname}->find(\$id);
		if(!\$row)
		{
			return redirect()
				->to('/{classroute}')
				->with('errors', 'Data is not found.');
		}
		return view('{classroute}/update', ['row'=>\$row]);
	}

	public function postUpdate(\$id = null)
	{
		\$row = \$this->{modelname}->find(\$id);
		if(!\$row)
		{
			return redirect()
				->to('/{classroute}')
				->with('errors', 'Data is not found.');
		}
		\$row->fill(\$this->request->getPost());
		if(!\$this->{modelname}->save(\$row))
		{
			return redirect()
				->to('/{classroute}/update/'.\$id)
				->withInput()
				->with('errors', \$this->{modelname}->errors());
		}
		return redirect()
			->to('/{classroute}')
			->with('success', 'Data has been saved.');
	}
}

EOT;
		// replace placeholders with variables
		$template = str_replace('{namespace}',  $namespace,  $template);
		$template = str_replace('{classname}',  $classname,  $template);
		$template = str_replace('{modelclass}', $modelclass, $template);
		$template = str_replace('{modelname}',  $modelname,  $template);
		$template = str_replace('{classroute}', $classroute,  $template);
		// if directory is not existed 
		if(!file_exists(APPPATH.'Controllers/'.$namespace.'/'))
		{
			mkdir(APPPATH.'Controllers/'.$namespace.'/', 0755, true);
			CLI::write('Created: ' . CLI::color('APPPATH/Controllers/'.$namespace.'/', 'green'));
		}
		// create the file
		$makefile = APPPATH.'Controllers/'.$namespace.'/'.$classname.'.php';
		file_put_contents($makefile, $template);
		CLI::write('Created: ' . CLI::color(str_replace(APPPATH, 'APPPATH/', $makefile), 'green'));

		//--------------------------------------------------------------------
		// Route template
		//--------------------------------------------------------------------
		$template = <<<EOT
// {classname}
\$routes->group('{classroute}', function(\$routes) {
	\$routes->setDefaultNamespace('App\Controllers\{namespace}');
	\$routes->get('/', '{classname}::index');
	\$routes->get('create', '{classname}::create');
	\$routes->post('create', '{classname}::postCreate');
	\$routes->get('show/(:num)', '{classname}::show/$1');
	\$routes->get('update/(:num)', '{classname}::update/$1');
	\$routes->post('update/(:num)', '{classname}::postUpdate/$1');
});

EOT;
		// replace placeholders with variables
		$template = str_replace('{namespace}',  $namespace,  $template);
		$template = str_replace('{classname}',  $classname,  $template);
		$template = str_replace('{classroute}', $classroute,  $template);
		// create the file
		$makefile = APPPATH.'Controllers/'.$namespace.'/Routes.php';
		// if file exist create and add it in Config/Routes.php
		if(!file_exists($makefile))
		{
			file_put_contents($makefile, '<?php'."\n", FILE_APPEND);
			file_put_contents(APPPATH.'Config/Routes.php', "require APPPATH.'Controllers/".$namespace."/Routes.php';\n", FILE_APPEND);
			CLI::write('Updated: ' . CLI::color(str_replace(APPPATH, 'APPPATH/', APPPATH.'Config/Routes.php'), 'green'));
		}

		file_put_contents($makefile, $template, FILE_APPEND);
		CLI::write('Updated: ' . CLI::color(str_replace(APPPATH, 'APPPATH/', $makefile), 'green'));

		//--------------------------------------------------------------------
		// AppLinks Config template
		//--------------------------------------------------------------------
		$template = <<<EOT
<?php namespace Config;

use CodeIgniter\Config\BaseConfig;

class AppLinks extends BaseConfig
{
	public \$links = [
	];
}

EOT;
		if(!file_exists(APPPATH.'Config/AppLinks.php'))
		{
			file_put_contents(APPPATH.'Config/AppLinks.php', $template);
			CLI::write('Created: ' . CLI::color(str_replace(APPPATH, 'APPPATH/', APPPATH.'Config/AppLinks.php'), 'green'));
		}

		//--------------------------------------------------------------------
		// AppLinks link template
		//--------------------------------------------------------------------
		$template = <<<EOT
		'{classroute}'=>['create'=>true, 'menu'=>true, 'namespace'=>'{namespace}', 'link'=>'{classroute}', 'name'=>'{classname}', 'icon'=>'<i class="fa fa-list"></i>'],
EOT;
		$template = str_replace('{namespace}',  $namespace,  $template);
		$template = str_replace('{classname}',  $classname,  $template);
		$template = str_replace('{classroute}', $classroute,  $template);
		// create the file
		$makefile = APPPATH.'Config/AppLinks.php';
		$AppLinks = file_get_contents($makefile);
		// append application link to $links array 
		$AppLinks = preg_replace("/(\s+];\s+}\\n)/", "\n".$template.'\\1', $AppLinks);
		file_put_contents(APPPATH.'Config/AppLinks.php', $AppLinks);
		CLI::write('Updated: ' . CLI::color(str_replace(APPPATH, 'APPPATH/', APPPATH.'Config/AppLinks.php'), 'green'));
		CLI::write('');

	}

}
