<?php namespace App\Commands;

use CodeIgniter\CLI\BaseCommand;
use CodeIgniter\CLI\CLI;

class MakeEntity extends BaseCommand
{
	protected $group = 'ErtikazOS';
	protected $name = 'make:entity';
	protected $description = 'Create a new entity class.';
	protected $usage = 'make:entity [namespace] [entity] [fields] or [options]';
	protected $arguments = [
		'namespace' => 'The namespace of your application.',
		'entity' => 'The name of the entity class.',
		'fields' => 'The list of fields separated by comma.',
	];
	protected $options = [
		'-t' => 'Set data table name',
	];

	public function run(array $params = [])
	{
		if (!isset($params[0]) || empty($params[0]))
		{
			CLI::error('"namespace" argument is missing.');
			CLI::write('');
			return;
		}
		if (!isset($params[1]) || empty($params[1]))
		{
			CLI::error('"entity" argument is missing.');
			CLI::write('');
			return;
		}

		// Class names must start with an uppercase letter.
		$namespace	= ucfirst(array_shift($params));
		$classname	= ucfirst(array_shift($params));
		$table		= CLI::getOption('t');

		// if table set but empty , show error
		if ($table == 1) 
		{
			CLI::error('"table" argument is missing.');
			CLI::write('');
			return;
		}
		// else if second argument in not set , show error
		elseif (empty($params[0]) && empty($table))
		{
			CLI::error('"fields" argument is missing.');
			CLI::write('');
			return;
		}

		// if table name set, retrieve table fields
		if(!empty($table) || $table == 1)
		{
			$db = \Config\Database::connect();
			if (!$db->tableExists($table))
			{
				CLI::error('"'.$table.'" table is not exists.');
				CLI::write('');
				return;
			}
			$fields = $db->getFieldData($table);

			foreach ($fields as $key => $field)
			{
				$fields[$key] = (array) $field;
			}
		} 
		else
		{
			$fields	= array_shift($params);
			$fields	= explode(',', $fields);
			foreach ($fields as $key => $field)
			{
				$field		= trim($field);
				$fields[$key] = ['name'=>$field, 'type'=>'', 'max_length'=>'', 'default'=>'', 'primary_key'=>0];
				if($field == 'id')
				{
					$fields[$key]['type'] = 'int';
					$fields[$key]['primary_key'] = 1;
				}
				elseif($field == 'created_at' || $field == 'updated_at')
				{
					$fields[$key]['type'] = 'datetime';
				}
			}
		}

		// Make $variables from $fields
		$variables	= [];
		$dates		= [];
		foreach ($fields as $key => $field)
		{
			// ignore empty field
			if(empty($field['name'])) continue;

			$variables[]	= "		'".$field['name']."' => null,";
			if($fields[$key]['type'] == 'datetime')
			{
				$dates[]	= "'".$field['name']."'";
			}
		}
		$variables	= implode("\n		", $variables);
		$dates		= implode(", ", $dates);

		//--------------------------------------------------------------------
		// entity template 
		//--------------------------------------------------------------------
		$template = <<<EOT
<?php namespace App\Entities\{namespace};

use CodeIgniter\Entity;

class {classname}Entity extends Entity
{
	protected \$attributes = [
		{variables}
	];
	protected \$datamap = [];
	protected \$dates = [{dates}];
	protected \$casts = [];
}

EOT;
		// replace placeholders with variables
		$template = str_replace('{namespace}', $namespace, $template);
		$template = str_replace('{classname}', $classname, $template);
		$template = str_replace('{variables}', $variables, $template);
		$template = str_replace('{dates}', $dates, $template);
		// if directory is not existed 
		if(!file_exists(APPPATH.'Entities/'.$namespace.'/'))
		{
			mkdir(APPPATH.'Entities/'.$namespace.'/', 0755, true);
			CLI::write('Created: ' . CLI::color('APPPATH/Entities/'.$namespace.'/', 'green'));
		}
		// create the file
		$makefile = APPPATH.'Entities/'.$namespace.'/'.$classname.'Entity.php';
		file_put_contents($makefile, $template);
		CLI::write('Created: ' . CLI::color(str_replace(APPPATH, 'APPPATH/', $makefile), 'green'));
		CLI::write('');
	}

}
