<?php namespace App\Commands;

use CodeIgniter\CLI\BaseCommand;
use CodeIgniter\CLI\CLI;

class MakeSeeder extends BaseCommand
{
	protected $group = 'ErtikazOS';
	protected $name = 'make:seeder';
	protected $description = 'Create a new seeder class from table.';
	protected $usage = 'make:seeder [table]';
	protected $arguments = [
		'name' => 'The table name.',
	];

	public function run(array $params = [])
	{
		if (!isset($params[0]) || empty($params[0]))
		{
			CLI::error('"name" argument is missing.');
			CLI::write('');
			return;
		}
		$name  = ucfirst(strtolower(array_shift($params)));
		$table = strtolower($name);
		$data  = [];
		$db    = \Config\Database::connect();
		if ($db->tableExists($table))
		{
			CLI::write(CLI::color('Table found file, fetch first 100 rows', 'green'));
			$query = $db->query("SELECT * FROM `$table` LIMIT 100");
			foreach ($query->getResultArray() as $row)
			{
				$data_row = [];
				foreach ($row as $key => $value)
				{
					$value		= str_replace("'", "\\'", $value);
					$data_row[] = "'$key' => '$value'";
				}
				$data[] = "			[\n				".implode(",\n				", $data_row)."\n			]";
			}
		}
		$data = trim(implode(",\n", $data));

		$template = <<<EOT
<?php namespace App\Database\Seeds;

class {name}Seeder extends \CodeIgniter\Database\Seeder
{
	public function run()
	{
		\$data = [
			{data}
		];
		\$this->db->table('{table}')->insertBatch(\$data);
	}
}

EOT;
		// replace placeholders with variables
		$template = str_replace('{name}', $name, $template);
		$template = str_replace('{table}', $table, $template);
		$template = str_replace('{data}', $data, $template);
		// create the file
		$makefile = APPPATH.'Database/Seeds/'.$name.'Seeder.php';
		file_put_contents($makefile, $template);
		CLI::write('Created file: ' . CLI::color($makefile, 'green'));
		CLI::write('');
	}

}
