<?php namespace App\Commands;

use CodeIgniter\CLI\BaseCommand;
use CodeIgniter\CLI\CLI;

class MakeView extends BaseCommand
{
	protected $group = 'ErtikazOS';
	protected $name = 'make:view';
	protected $description = 'Create a new view file from model class.';
	protected $usage = 'make:view [namespace] [controller] [model]';
	protected $arguments = [
		'namespace' => 'The namespace of your application.',
		'controller' => 'The name of the controller class.',
		'model' => 'The name of the model class.',
	];

	public function run(array $params = [])
	{
		if (!isset($params[0]) || empty($params[0]))
		{
			CLI::error('"namespace" argument is missing.');
			CLI::write('');
			return;
		}
		if (!isset($params[1]) || empty($params[1]))
		{
			CLI::error('"controller" argument is missing.');
			CLI::write('');
			return;
		}
		if (!isset($params[2]) || empty($params[2]))
		{
			CLI::error('"model" argument is missing.');
			CLI::write('');
			return;
		}

		// Class names must start with an uppercase letter.
		$namespace  = ucfirst(array_shift($params));
		$classname  = ucfirst(array_shift($params));
		$modelclass = ucfirst(array_shift($params));
		// model variable name 
		$modelname  = strtolower($modelclass);
		// used in views
		$classroute = str_replace('\\', '/', $namespace.'/'.$classname);
		$classroute = strtolower(trim($classroute, '/'));

		$modelfile = APPPATH.'Models/'.$namespace.'/'.$modelclass.'.php';
		if(!file_exists($modelfile))
		{
			CLI::error('model file: '.$modelfile.' is not exist.');
			CLI::write('');
			return;
		}
		require_once($modelfile);
		$class  = 'App\Models\\'.$namespace.'\\'.$modelclass;
		if(class_exists($modelclass))
		{
			CLI::error('model class: '.$class.' is not exist.');
			CLI::write('');
			return;
		}

		$object = new $class();
		$rules  = $object->getValidationRules();
		if(!file_exists(APPPATH.'Views/'.strtolower($namespace.'/'.$classname).'/'))
		{
			mkdir(APPPATH.'Views/'.strtolower($namespace.'/'.$classname).'/', 0755, true);
			CLI::write('Created: ' . CLI::color('APPPATH/Views/'.strtolower($namespace.'/'.$classname).'/', 'green'));
		}

		//--------------------------------------------------------------------
		// make index view
		//--------------------------------------------------------------------
$template = <<<EOT
<?php echo \$this->extend('layout') ?>
<?php echo \$this->section('stylesheet') ?>
<?php echo \$this->endSection() ?>
<?php echo \$this->section('content') ?>
<div class="row">
	<div class="col-12">
		<div class="card">
			<div class="card-body">
				<table class="table table-bordered table-responsive">
					<thead>
						<tr>
						{theads}
						</tr>
					</thead>
					<tbody>
						<?php foreach (\$rows as \$key => \$row) {?>
						<tr>
						{tbodys}
						</tr>
						<?php } ?>
					</tbody>
				</table>
				<?php echo \$pager->links('{modelname}') ?>
			</div>
		</div>
	</div>
</div>
<?php echo \$this->endSection() ?>
<?php echo \$this->section('javascript') ?>
<?php echo \$this->endSection() ?>

EOT;
		$theads = [];
		$tbodys = [];
		foreach ($rules as $name => $rule)
		{
			$theads[] = '    <th scope="col">'.ucfirst($name).'</th>';
			$tbodys[] = '    <td scope="row"><?php echo $row->'.$name.'?></td>';
		}
		// add action column
		$theads[] = '    <th scope="col">Actions</th>';
		$tbodys[] = '    <td>';
		$tbodys[] = '        <a class="btn btn-primary" href="<?php echo base_url()?>{classroute}/show/<?php echo $row->id?>"><i class="fa fa-bars"></i></a>';
		$tbodys[] = '        <a class="btn btn-warning" href="<?php echo base_url()?>{classroute}/update/<?php echo $row->id?>"><i class="fa fa-edit"></i></a>';
		$tbodys[] = '    </td>';

		$theads = implode("\n                        ", $theads);
		$tbodys = implode("\n                        ", $tbodys);
		// replace placeholders with variables
		$template = str_replace('{theads}', $theads, $template);
		$template = str_replace('{tbodys}', $tbodys, $template);
		$template = str_replace('{modelname}', $modelname, $template);
		$template = str_replace('{classroute}', $classroute, $template);
		// create the file
		$makefile = APPPATH.'Views/'.strtolower($namespace.'/'.$classname).'/index.php';
		file_put_contents($makefile, $template);
		CLI::write('Created: ' . CLI::color(str_replace(APPPATH, 'APPPATH/', $makefile), 'green'));


		//--------------------------------------------------------------------
		// make show view
		//--------------------------------------------------------------------
$template = <<<EOT
<?php echo \$this->extend('layout') ?>
<?php echo \$this->section('stylesheet') ?>
<?php echo \$this->endSection() ?>
<?php echo \$this->section('content') ?>
<div class="row">
	<div class="col-12">
		<div class="card">
			<div class="card-body">
				{fields}
				<a class="btn btn-warning" href="<?php echo base_url()?>{classroute}/update/<?php echo \$row->id?>"><i class="fa fa-edit"></i> Edit</a>
			</div>
		</div>
	</div>
</div>
<?php echo \$this->endSection() ?>
<?php echo \$this->section('javascript') ?>
<?php echo \$this->endSection() ?>

EOT;
		$fields = [];
		foreach ($rules as $name => $rule)
		{
			$fields[] = '<div class="form-group row">';
			$fields[] = '    <label class="control-label col-md-2">'.ucfirst($name).'</label>';
			$fields[] = '    <div class="col-md-10">';
			$fields[] = '        <p class="form-control-static"><?php echo $row->'.$name.'?></p>';
			$fields[] = '    </div>';
			$fields[] = '</div>';
		}

		$fields = implode("\n                ", $fields);
		// replace placeholders with variables
		$template = str_replace('{fields}', $fields, $template);
		$template = str_replace('{classroute}', $classroute, $template);
		// create the file
		$makefile = APPPATH.'Views/'.strtolower($namespace.'/'.$classname).'/show.php';
		file_put_contents($makefile, $template);
		CLI::write('Created: ' . CLI::color(str_replace(APPPATH, 'APPPATH/', $makefile), 'green'));




		//--------------------------------------------------------------------
		// make create view
		//--------------------------------------------------------------------
		$template = <<<EOT
<?php echo \$this->extend('layout') ?>
<?php echo \$this->section('stylesheet') ?>
<?php echo \$this->endSection() ?>
<?php echo \$this->section('content') ?>
<div class="row">
	<div class="col-12">
		<div class="card">
			<div class="card-body">
				<form action="<?php echo base_url()?>{classroute}/create" method="POST">
					{fields}
					<button type="submit" class="btn btn-primary">Save</button>
				</form>
			</div>
		</div>
	</div>
</div>
<?php echo \$this->endSection() ?>
<?php echo \$this->section('javascript') ?>
<?php echo \$this->endSection() ?>

EOT;
		$fields = [];
		foreach ($rules as $name => $rule)
		{
			$fields[] = '<div class="form-group row">';
			$fields[] = '   <label for="'.$name.'" class="col-md-2 col-form-label">'.ucfirst($name).'</label>';
			$fields[] = '   <div class="col-md-10">';
			$fields[] = '       <input type="text" class="form-control" id="'.$name.'" name="'.$name.'" aria-describedby="'.$name.'Help" placeholder="Enter '.ucfirst($name).'" value="<?php echo old(\''.$name.'\')?>">';
			$fields[] = '       <?php if(isset(session(\'errors\')[\''.$name.'\'])){ ?>';
			$fields[] = '       <small id="'.$name.'Help" class="form-text text-danger"><?php echo session(\'errors\')[\''.$name.'\']?></small>';
			$fields[] = '       <?php } ?>';
			$fields[] = '       <small id="'.$name.'Help" class="form-text text-muted">please enter '.ucfirst($name).'</small>';
			$fields[] = '   </div>';
			$fields[] = '</div>';
			$fields[] = '';
		}
		$fields = implode("\n                    ", $fields);
		// replace placeholders with variables
		$template = str_replace('{fields}', $fields, $template);
		$template = str_replace('{classroute}', $classroute, $template);
		// create the file
		$makefile = APPPATH.'Views/'.strtolower($namespace.'/'.$classname).'/create.php';
		file_put_contents($makefile, $template);
		CLI::write('Created: ' . CLI::color(str_replace(APPPATH, 'APPPATH/', $makefile), 'green'));

		//--------------------------------------------------------------------
		// make update view
		//--------------------------------------------------------------------
		$template = <<<EOT
<?php echo \$this->extend('layout') ?>
<?php echo \$this->section('stylesheet') ?>
<?php echo \$this->endSection() ?>
<?php echo \$this->section('content') ?>
<div class="row">
	<div class="col-12">
		<div class="card">
			<div class="card-body">
				<form action="<?php echo base_url()?>{classroute}/update/<?php echo \$row->id?>" method="POST">
					{fields}
					<button type="submit" class="btn btn-primary">Update</button>
				</form>
			</div>
		</div>
	</div>
</div>
<?php echo \$this->endSection() ?>
<?php echo \$this->section('javascript') ?>
<?php echo \$this->endSection() ?>

EOT;
		$fields = [];
		foreach ($rules as $name => $rule)
		{
			$fields[] = '<div class="form-group row">';
			$fields[] = '   <label for="'.$name.'" class="col-md-2 col-form-label">'.ucfirst($name).'</label>';
			$fields[] = '   <div class="col-md-10">';
			$fields[] = '       <input type="text" class="form-control" id="'.$name.'" name="'.$name.'" aria-describedby="'.$name.'Help" placeholder="Enter '.ucfirst($name).'" value="<?php echo old(\''.$name.'\', $row->'.$name.')?>">';
			$fields[] = '       <?php if(isset(session(\'errors\')[\''.$name.'\'])){ ?>';
			$fields[] = '       <small id="'.$name.'Help" class="form-text text-danger"><?php echo session(\'errors\')[\''.$name.'\']?></small>';
			$fields[] = '       <?php } ?>';
			$fields[] = '       <small id="'.$name.'Help" class="form-text text-muted">please enter '.ucfirst($name).'</small>';
			$fields[] = '   </div>';
			$fields[] = '</div>';
			$fields[] = '';
		}
		$fields = implode("\n                    ", $fields);
		// replace placeholders with variables
		$template = str_replace('{fields}', $fields, $template);
		$template = str_replace('{classroute}', $classroute, $template);
		// create the file
		$makefile = APPPATH.'Views/'.strtolower($namespace.'/'.$classname).'/update.php';
		file_put_contents($makefile, $template);
		CLI::write('Created: ' . CLI::color(str_replace(APPPATH, 'APPPATH/', $makefile), 'green'));
		CLI::write('');

	}

}
