<?php namespace App\Commands;

use CodeIgniter\CLI\BaseCommand;
use CodeIgniter\CLI\CLI;

class StoreAccount extends BaseCommand
{
	protected $token = 'https://store.ertikazos.com/token/';
	protected $group = 'ErtikazOS';
	protected $name = 'store:account';
	protected $description = 'Create, reset and set account in ErtikazOS store.';
	protected $usage = 'store:account [action] [email] name | code';
	protected $arguments = [
		'action' => 'The account action create | reset | set.',
		'email' => 'The account email.',
		'name' => 'The account name | code.',
	];

	public function run(array $params = [])
	{
		if (!isset($params[0]) || empty($params[0]))
		{
			CLI::error('"action" argument is missing.');
			CLI::write('');
			return;
		}

		// action parameter
		$action = strtolower(array_shift($params));
		if ($action != 'create' && $action != 'reset' && $action != 'set')
		{
			CLI::error('"action" aaargument must equal create | reset | set.');
			CLI::write('');
			return;
		}

		if (!isset($params[0]) || empty($params[0]))
		{
			CLI::error('"email" argument is missing.');
			CLI::write('');
			return;
		}

		// email parameter
		$email = strtolower(array_shift($params));
		if (filter_var($email, FILTER_VALIDATE_EMAIL) === FALSE)
		{
			CLI::error('"email" argument is invalid email address.');
			CLI::write('');
			return;
		}

		// create new account
		if($action == 'create')
		{
			$name = array_shift($params);
			if (empty($name))
			{
				CLI::error('account "name" is missing.');
				CLI::write('');
				return;
			}
			$post = ['email'=>$email, 'name'=>$name];
			$data = $this->_post($post, $action);
			if($data->code != 'success')
			{
				CLI::error($data->message);
				CLI::write('');
				return;
			}
			if(isset($data->token))
			{
				CLI::write('Account has been created.');
				CLI::write('Account token is: '.$data->token.' .');
				if(file_put_contents(WRITEPATH.'.account-token', $data->token))
				{
					CLI::write('Token saved to: '.substr(WRITEPATH.'.account-token', strlen(ROOTPATH)));
				}
				else
				{
					CLI::error('Token cannot saved to: '.substr(WRITEPATH.'.account-token', strlen(ROOTPATH)));
				}
			}
		}

		// reset account token
		if($action == 'reset')
		{
			$post = ['email'=>$email];
			$data = $this->_post($post, $action);
			if($data->code != 'success')
			{
				CLI::error($data->message);
				CLI::write('');
				return;
			}
			CLI::write($data->message);
			CLI::write('');
		}

		// set account token
		if($action == 'set')
		{
			$code = array_shift($params);
			if (empty($code))
			{
				CLI::error('account code is missing.');
				CLI::write('');
				return;
			}
			$post = ['email'=>$email, 'code'=>$code];
			$data = $this->_post($post, $action);
			if($data->code != 'success')
			{
				CLI::error($data->message);
				CLI::write('');
				return;
			}
			if(isset($data->token))
			{
				CLI::write('Account has been reseted.');
				CLI::write('Account token is: '.$data->token.' .');
				if(file_put_contents(WRITEPATH.'.account-token', $data->token))
				{
					CLI::write('Token saved to: '.substr(WRITEPATH.'.account-token', strlen(ROOTPATH)));
				}
				else
				{
					CLI::error('Token cannot saved to: '.substr(WRITEPATH.'.account-token', strlen(ROOTPATH)));
				}
				CLI::write('');
			}
		}
	}

	/**
	 *
	 * post data to store token URL.
	 *
	 */
	public function _post($post, $action)
	{
		$ch = curl_init($this->token.$action);
		curl_setopt($ch, CURLOPT_POST, true);
		curl_setopt($ch, CURLOPT_POSTFIELDS, $post);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		$data = curl_exec($ch);
		curl_close($ch);
		$data = json_decode($data);
		if(json_last_error() == JSON_ERROR_NONE)
		{
			return $data;
		}
		CLI::error('invalid response from server, please try again.');
		CLI::write('');
		return;
	}
}
