<?php namespace App\Commands;

use CodeIgniter\CLI\BaseCommand;
use CodeIgniter\CLI\CLI;

class StoreJson extends BaseCommand
{
	protected $group = 'ErtikazOS';
	protected $name = 'store:json';
	protected $description = 'Create the package info.json file for ErtikazOS store.';
	protected $usage = 'store:json [app]';
	protected $arguments = [
		'app' => 'The application name.',
	];

	public function run(array $params = [])
	{
		if (!isset($params[0]) || empty($params[0]))
		{
			CLI::error('"app" argument is missing.');
			CLI::write('');
			return;
		}

		// Class names must start with an uppercase letter.
		$App = ucfirst(array_shift($params));
		if(!file_exists(APPPATH.'Controllers/'.$App.'/'))
		{
			CLI::error('"'.'Controllers/'.$App.'/'.'" is not existed.');
			CLI::write('');
			return;
		}

		if(file_exists(APPPATH.'Controllers/'.$App.'/info.json'))
		{
			CLI::error('"'.'Controllers/'.$App.'/info.json'.'" is already existed.');
			CLI::write('');
			return;
		}

		// start creating JSON object
		$info = new \stdClass;
		$info->name = $App;
		$info->description = $App.' description';
		$info->version = '1.0.0';
		$info->keywords = '';

		// the package author information 
		$info->authors = [];
		$info->authors[0] = new \stdClass;
		$info->authors[0]->name = 'author name';
		$info->authors[0]->email = 'author email';
		$info->authors[0]->homepage = 'author homepage';
		$info->authors[0]->role = 'author role';

		// the package requirement 
		$info->require = new \stdClass;
		$info->require->CodeIgniter = '4.0.0-beta.3';
		$info->require->php = '7.2.0';
		$info->price = '0';
		$info->license = 'GPL-3.0+';
		// package files
		$info->files = [];

		// define your application filters
		$info->filters = [];
		$info->filters[0] = new \stdClass;
		// any filter name, 'isAuthenticated' will prevent any unauthorized user from access these routes.
		$info->filters[0]->filter = 'isAuthenticated';
		// execute this filter before of after.
		$info->filters[0]->when = 'before';
		// an array of path
		$info->filters[0]->routes = [$App.'/*'];

		// define links
		$info->links = [];
		$info->links[0] = new \stdClass;
		// show/hide create button
		$info->links[0]->create = 1;
		// show/hide link in side menu
		$info->links[0]->menu = 1;
		// the application link URL
		$info->links[0]->link = $App;
		// the application link label
		$info->links[0]->name = $App;
		// the application link icon
		$info->links[0]->icon = '<i class="fa fa-list"></i>';
		// the current time
		$info->time = gmdate('YmdHis');

		// find any package files
		$Controllers	= $this->findFiles(APPPATH.'Controllers/'.$App);
		$Models			= $this->findFiles(APPPATH.'Models/'.$App,'[.+\.php]');
		$Entities		= $this->findFiles(APPPATH.'Entities/'.$App,'[.+\.php]');
		$Views			= $this->findFiles(APPPATH.'Views/'.strtolower($App));
		$assets			= $this->findFiles(FCPATH .'assets/'.strtolower($App));
		$AppFiles		= array_merge($Controllers, $Models, $Entities, $Views, $assets);
		foreach ($AppFiles as $key => $file)
		{
			$info->files[] = substr($file, strlen(ROOTPATH));
		}

		// Removes duplicate values
		$info->files = array_unique($info->files);

		// Sort array files 
		asort($info->files);

		// Append files and write file again 
		file_put_contents(APPPATH.'Controllers/'.$App.'/info.json', json_encode($info, JSON_PRETTY_PRINT));

		CLI::write('Create: '.'Controllers/'.$App.'/info.json');

		CLI::write('');
	}

	public function findFiles($folder, $pattern='[.+]')
	{
		if(!file_exists($folder) || !is_dir($folder))
		{
			return [];
		}
		$fileList = [];
		$dir = new \RecursiveDirectoryIterator($folder);
		$ite = new \RecursiveIteratorIterator($dir);
		$files = new \RegexIterator($ite, $pattern, \RegexIterator::GET_MATCH);
		foreach($files as $file)
		{
			if(substr($file[0], -3) == '/..' || substr($file[0], -2) == '/.')
			{
				continue;
			}
			$fileList = array_merge($fileList, $file);
		}
		return $fileList;
	}
}
