<?php namespace App\Commands;

use CodeIgniter\CLI\BaseCommand;
use CodeIgniter\CLI\CLI;

class StorePush extends BaseCommand
{
	protected $store = 'https://store.ertikazos.com/upload/';
	protected $group = 'ErtikazOS';
	protected $name = 'store:push';
	protected $description = 'Pack and push package to ErtikazOS store.';
	protected $usage = 'store:push [app]';
	protected $arguments = [
		'app' => 'The application name.',
	];

	public function run(array $params = [])
	{
		if (!isset($params[0]) || empty($params[0]))
		{
			CLI::error('"namespace" argument is missing.');
			CLI::write('');
			return;
		}

		// Class names must start with an uppercase letter.
		$App = ucfirst(array_shift($params));
		if(!file_exists(APPPATH.'Controllers/'.$App.'/info.json'))
		{
			CLI::error('"'.'Controllers/'.$App.'/info.json'.'" is missing.');
			CLI::write('');
			return;
		}
		$info			= file_get_contents(APPPATH.'Controllers/'.$App.'/info.json');
		$info			= json_decode($info);
		if(json_last_error() != JSON_ERROR_NONE)
		{
			CLI::error('"'.'Controllers/'.$App.'/info.json'.'" cannot be decoded.');
			CLI::write('');
			return;
		}
		$Controllers	= $this->findFiles(APPPATH.'Controllers/'.$App);
		$Models			= $this->findFiles(APPPATH.'Models/'.$App,'[.+\.php]');
		$Entities		= $this->findFiles(APPPATH.'Entities/'.$App,'[.+\.php]');
		$Views			= $this->findFiles(APPPATH.'Views/'.strtolower($App));
		$assets			= $this->findFiles(FCPATH .'assets/'.strtolower($App));
		$AppFiles		= array_merge($Controllers, $Models, $Entities, $Views, $assets);
		foreach ($AppFiles as $key => $file)
		{
			$info->files[] = substr($file, strlen(ROOTPATH));
		}

		// Removes duplicate values
		$info->files = array_unique($info->files);

		// Sort array files 
		asort($info->files);

		// remove array key
		$info->files = array_values($info->files);

		// set time GMT 
		$info->time = gmdate('YmdHis');

		// Append files and write file again 
		file_put_contents(APPPATH.'Controllers/'.$App.'/info.json', json_encode($info, JSON_PRETTY_PRINT));
		$package_file = WRITEPATH.$App.'-'.$info->version.'.zip';
		if(file_exists($package_file))
		{
			CLI::error('package file is already existed: '.$package_file);
			CLI::write('');
			return;
		}

		// create new Zip Archive
		$zip = new \ZipArchive;
		$res = $zip->open($package_file, \ZipArchive::CREATE);
		if ($res === TRUE)
		{
			CLI::write('Create: '.substr($package_file, strlen(ROOTPATH)));
			$zip->setArchiveComment('ErtikazOS '.$App.' package file.');
			foreach ($info->files as $key => $file)
			{
				if(!file_exists(ROOTPATH.$file))
				{
					CLI::error('Error file '.$file.' is missing.');
					CLI::write('');
					return;
				}
				$zip->addFile(ROOTPATH.$file, $file);
				CLI::write('   add: '.$file);
			}
			$zip->close();
		}
		else
		{
			CLI::error('Error while open '.$package_file.' file.');
			CLI::write('');
			return;
		}
		if(!file_exists(WRITEPATH.'.account-token'))
		{
			CLI::error('Account token is not found, create one using "spark store:token create $name $mail".');
			CLI::write('');
			return;
		}
		CLI::write('Uploading '.$App.'-'.$info->version.' package...');
		$data = $this->_PushPackage($package_file);
		CLI::write('');
		if(is_object($data) && isset($data->code) && isset($data->message))
		{
			if($data->code != 'success')
			{
				CLI::error($data->message);
				CLI::write('');
				return;
			}
			CLI::write($data->message);
		}
		else
		{
			CLI::error('Invalid response from the server, try again later.');
			CLI::write('');
			return;
		}
		CLI::write('');
	}

	public function findFiles($folder, $pattern='[.+]')
	{
		if(!file_exists($folder) || !is_dir($folder))
		{
			return [];
		}
		$fileList = [];
		$dir = new \RecursiveDirectoryIterator($folder);
		$ite = new \RecursiveIteratorIterator($dir);
		$files = new \RegexIterator($ite, $pattern, \RegexIterator::GET_MATCH);
		foreach($files as $file)
		{
			if(substr($file[0], -3) == '/..' || substr($file[0], -2) == '/.')
			{
				continue;
			}
			$fileList = array_merge($fileList, $file);
		}
		return $fileList;
	}

	/**
	*
	* Push package to ErtikazOS store 
	*
	*/
	public function _PushPackage($package_file)
	{
		helper('number');
		$token = file_get_contents(WRITEPATH.'.account-token');
		$file = curl_file_create($package_file, 'application/zip', basename($package_file));
		$post = ['package' => $file, 'token'=>$token];
		$link = $this->store.str_replace(['.zip','-'],['','/'],basename($package_file));
		$ch = curl_init($link);
		curl_setopt($ch, CURLOPT_POST, true);
		curl_setopt($ch, CURLOPT_POSTFIELDS, $post);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_NOPROGRESS, false);
		curl_setopt($ch, CURLOPT_PROGRESSFUNCTION, function($ch, $download_size = 0, $downloaded = 0, $upload_size = 0, $uploaded = 0){
			if($uploaded > 0)
			{
				$percent    = floor(($uploaded / $upload_size) * 100);
			} else {
				$percent    = 0;
			}
			$left       = 100 - $percent;
			$write      = sprintf("\033[0G\033[2K[%'={$percent}s>%-{$left}s] - $percent%% - ".number_to_size($uploaded)."/".number_to_size($upload_size), "", "");
			fwrite(STDERR, $write);
		});
		$data = curl_exec($ch);
		curl_close($ch);
		CLI::write('');
		$data = json_decode($data);
		if(json_last_error() == JSON_ERROR_NONE)
		{
			return $data;
		}
	}
}
