<?php namespace Config;

use CodeIgniter\Config\BaseConfig;

class Auth extends BaseConfig
{

	/*
	|--------------------------------------------------------------------------
	| Authentication Driver
	|--------------------------------------------------------------------------
	|
	| The driver to use:
	|	\App\Controllers\Auth\Drivers\DatabaseDriver
	|	\App\Controllers\Auth\Drivers\EwsDriver
	|	\App\Controllers\Auth\Drivers\ImapDriver
	|	\App\Controllers\Auth\Drivers\LdapDriver
	|
	| The Default is:
	|	\App\Controllers\Auth\Drivers\DatabaseDriver
	|
	*/
	public $driver = '\App\Controllers\Auth\Drivers\DatabaseDriver';

	/*
	|--------------------------------------------------------------------------
	| Default options for new logged-in users
	|--------------------------------------------------------------------------
	|
	| This variable allow system to set the default applications
	|
	*/
	public $options = ['apps'=> [
			'user' => 1,
			'user/profile' => 1
		]
	];

	/*
	|--------------------------------------------------------------------------
	| EwsDriver Variables
	|--------------------------------------------------------------------------
	|
	| 'ews_url'
	|
	|	The exchange web service full URL.
	|
	| 'ews_domain'
	|
	|	The domain name.
	|
	*/
	public $ews_url = 'https://mail.example.com/EWS/exchange.asmx';
	public $ews_domain = 'example.com';

	/*
	|--------------------------------------------------------------------------
	| ImapDriver Variables
	|--------------------------------------------------------------------------
	|
	| 'imap_host'
	|
	|	The IMAP server IP or host.
	|
	| 'imap_port'
	|
	|	The IMAP server port.
	|
	| 'imap_domain'
	|
	|	The IMAP domain.
	|
	| 'imap_flags'
	|
	|	The IMAP optional flags, see https://php.net/imap_open
	|
	*/
	public $imap_host = 'imap.example.com';
	public $imap_port = '993';
	public $imap_domain = 'example.com';
	public $imap_flags = '/imap/ssl/novalidate-cert';

	/*
	|--------------------------------------------------------------------------
	| LdapDriver Variables
	|--------------------------------------------------------------------------
	|
	| 'ldap_host'
	|
	|	The LDAP server IP or host.
	|
	| 'ldap_port'
	|
	|	The LDAP server port.
	|
	| 'ldap_domain'
	|
	|	The LDAP domain.
	|
	| 'ldap_base_dn'
	|
	|	The base DN for the directory.
	|
	| 'ldap_sam_account_name'
	|
	|	The user identifier attribute, Default: SamAccountName.
	|
	| 'ldap_allowed_ous'
	|
	|	Allow only the users from these OUs.
	|
	*/
	public $ldap_host = 'ldap.example.com';
	public $ldap_port = '389';
	public $ldap_domain = 'example.com';
	public $ldap_base_dn = 'dc=example,dc=com';
	public $ldap_sam_account_name = 'SamAccountName';
	public $ldap_allowed_ous = '';

}
