<?php namespace App\Controllers\Admin;

use Config\Services;
use App\Models\Lookup;
use App\Models\Admin\User;
use App\Entities\Admin\UserEntity;
use App\Controllers\BaseController;

class Users extends BaseController
{
    protected $lookup;
    protected $user;
    function __construct()
    {
        $this->lookup = new Lookup();
        $this->user = new User();
    }

    public function index()
    {
        $limit  = 10;
        $offset = 0;
        $user = $this->user;
        $rows = $user->paginate($limit, 'user');
        return view('admin/users/index', ['rows'=>$rows, 'pager'=>$user->pager]);
    }

    public function show($id = null)
    {
        $row = $this->user->find($id);
        if(!$row)
        {
            return redirect()
                ->to('/admin/users')
                ->with('errors', 'Data is not found.');
        }
        return view('admin/users/show', ['row'=>$row]);
    }

    public function create()
    {
        $statuses = $this->lookup->status();
        $routes = Services::routes()->getRoutes();
        return view('admin/users/create',['statuses'=>$statuses, 'routes'=>$routes]);
    }

    public function postCreate()
    {
        $userEntity = new UserEntity();
        $userEntity->fill($this->request->getPost());
        if(!$this->user->save($userEntity))
        {
            return redirect()
                ->to('/admin/users/create')
                ->withInput()
                ->with('errors', $this->user->errors());
        }
        return redirect()
            ->to('/admin/users')
            ->with('success', 'Data has been saved.');
    }

    public function update($id = null)
    {
        $row = $this->user->find($id);
        if(!$row)
        {
            return redirect()
                ->to('/admin/users')
                ->with('errors', 'Data is not found.');
        }
        $statuses = $this->lookup->status();
        $routes = Services::routes()->getRoutes();
        return view('admin/users/update', ['row'=>$row,'statuses'=>$statuses, 'routes'=>$routes]);
    }

    public function postUpdate($id = null)
    {
        $row = $this->user->find($id);
        if(!$row)
        {
            return redirect()
                ->to('/admin/users')
                ->with('errors', 'Data is not found.');
        }
        $row->fill($this->request->getPost());
        if(!$this->user->save($row))
        {
            return redirect()
                ->to('/admin/users/update/'.$id)
                ->withInput()
                ->with('errors', $this->user->errors());
        }
        return redirect()
            ->to('/admin/users')
            ->with('success', 'Data has been saved.');
    }
}
