<?php namespace App\Controllers\Auth\Drivers;

/**
 * Auth interface
 */

interface AuthInterface {

	/**
	 * User login.
	 *
	 * @param   string   $email     user email
	 * @param   string   $password  user password
	 * @param   boolean  $remember  Enable autologin
	 *
	 * @return  \App\Entities\Auth\UserEntity | \CodeIgniter\HTTP\RedirectResponse
	 */
	public function login($email, $password, bool $remember = false);

	//--------------------------------------------------------------------

	/**
	 * User Register.
	 *
	 * @param   string   $email     user email
	 * @param   string   $name      user name
	 * @param   string   $mobile    user mobile
	 * @param   string   $password  user password
	 *
	 * @return  boolean
	 */
	public function register($email, $name, $mobile, $password);

	//--------------------------------------------------------------------

	/**
	 * User Lost.
	 *
	 * @param   string   $email     user email
	 *
	 * @return  boolean
	 */
	public function lost($email);

	//--------------------------------------------------------------------

	/**
	 * User Reset.
	 *
	 * @param   string   $email     user email
	 * @param   string   $code      user code
	 * @param   string   $password  user password
	 *
	 * @return  boolean
	 */
	public function reset($email, $code, $password);

	//--------------------------------------------------------------------

	/**
	 * User logout.
	 *
	 * @param   string   $email     user email
	 *
	 * @return  boolean
	 */
	public function logout($email);

	//--------------------------------------------------------------------
}
