<?php namespace App\Controllers\Auth\Drivers;

use App\Models\Auth\User;
use App\Entities\Auth\UserEntity;

/**
 * Authenticate users using database
 */

class DatabaseDriver implements AuthInterface {

	/**
	 * User login.
	 *
	 * @param   string   $email     user email
	 * @param   string   $password  user password
	 * @param   boolean  $remember  Enable autologin
	 *
	 * @return  \App\Entities\Auth\UserEntity | \CodeIgniter\HTTP\RedirectResponse
	 */
	public function login($email, $password, bool $remember = false)
	{
		$user = new User();
		$userEntity = $user->where('email', $email)->first();

		if (! $userEntity instanceof UserEntity)
		{
		   return redirect()
				->to('/auth/login')
				->with('errors', lang('Auth.wrong_user'));
		}

		if (! password_verify($password, $userEntity->password))
		{
			return redirect()
				->to('/auth/login')
				->with('errors', lang('Auth.wrong_user'));
		}

		return $userEntity;
	}

	//--------------------------------------------------------------------

	/**
	 * User Register.
	 *
	 * @param   string   $email     user email
	 * @param   string   $name      user name
	 * @param   string   $mobile    user mobile
	 * @param   string   $password  user password
	 *
	 * @return  boolean
	 */
	public function register($email, $name, $mobile, $password)
	{

	}

	//--------------------------------------------------------------------

	/**
	 * User Lost.
	 *
	 * @param   string   $email     user email
	 *
	 * @return  boolean
	 */
	public function lost($email)
	{

	}

	//--------------------------------------------------------------------

	/**
	 * User Reset.
	 *
	 * @param   string   $email     user email
	 * @param   string   $code      user code
	 * @param   string   $password  user password
	 *
	 * @return  boolean
	 */
	public function reset($email, $code, $password)
	{

	}

	//--------------------------------------------------------------------

	/**
	 * User logout.
	 *
	 * @param   string   $email     user email
	 *
	 * @return  boolean
	 */
	public function logout($email)
	{

	}

	//--------------------------------------------------------------------
}
