<?php namespace App\Controllers\Auth\Drivers;

use App\Models\Auth\User;
use App\Entities\Auth\UserEntity;

/**
 * Authenticate users using EWS
 */

class EwsDriver implements AuthInterface {

	/**
	 * User login.
	 *
	 * @param   string   $email     user email
	 * @param   string   $password  user password
	 * @param   boolean  $remember  Enable autologin
	 *
	 * @return  \App\Entities\Auth\UserEntity | \CodeIgniter\HTTP\RedirectResponse
	 */
	public function login($email, $password, bool $remember = false)
	{
		$options	= config('Auth')->options;
		$ews_url	= config('Auth')->ews_url;
		$ews_domain	= config('Auth')->ews_domain;

		list($username, ) = explode('@', $email, 2);

		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $ews_url);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
		curl_setopt($ch, CURLOPT_HEADER, TRUE);
		curl_setopt($ch, CURLOPT_HTTPAUTH, CURLAUTH_NTLM);
		curl_setopt($ch, CURLOPT_USERPWD, $username.':'.$password);
		$data = curl_exec($ch);
		if(curl_getinfo($ch, CURLINFO_HTTP_CODE) != '200')
		{
			return redirect()
				->to('/auth/login')
				->with('errors', lang('Auth.wrong_user'));
		}

		$user = new User();
		$userEntity = $user->where('email', $email)->first();
		if (! $userEntity instanceof UserEntity)
		{
			$UserEntity = new UserEntity();
			$UserEntity->name = $username;
			$UserEntity->email = $email;
			$UserEntity->password = password_hash('12345678', PASSWORD_DEFAULT);
			$UserEntity->mobile = '0';
			$UserEntity->options = $options;
			$UserEntity->authcode = '';
			$UserEntity->status = 'On';
			$UserEntity->created_by = '0';
			$UserEntity->created_at = date('Y-m-d H:i:s');
			$user->save($UserEntity);
			// Get user row
			$userEntity = $user->where('email', $email)->first();
		}

		if (! $userEntity instanceof UserEntity)
		{
			return redirect()
				->to('/auth/login')
				->with('errors', lang('Auth.wrong_user'));
		}

		return $userEntity;
	}

	//--------------------------------------------------------------------

	/**
	 * User Register.
	 *
	 * @param   string   $email     user email
	 * @param   string   $name      user name
	 * @param   string   $mobile    user mobile
	 * @param   string   $password  user password
	 *
	 * @return  boolean
	 */
	public function register($email, $name, $mobile, $password)
	{

	}

	//--------------------------------------------------------------------

	/**
	 * User Lost.
	 *
	 * @param   string   $email     user email
	 *
	 * @return  boolean
	 */
	public function lost($email)
	{

	}

	//--------------------------------------------------------------------

	/**
	 * User Reset.
	 *
	 * @param   string   $email     user email
	 * @param   string   $code      user code
	 * @param   string   $password  user password
	 *
	 * @return  boolean
	 */
	public function reset($email, $code, $password)
	{

	}

	//--------------------------------------------------------------------

	/**
	 * User logout.
	 *
	 * @param   string   $email     user email
	 *
	 * @return  boolean
	 */
	public function logout($email)
	{

	}

	//--------------------------------------------------------------------
}
