<?php namespace App\Controllers\Auth\Drivers;

use App\Models\Auth\User;
use App\Entities\Auth\UserEntity;

/**
 * Authenticate users using IMAP
 */

class ImapDriver implements AuthInterface {

	/**
	 * User login.
	 *
	 * @param   string   $email     user email
	 * @param   string   $password  user password
	 * @param   boolean  $remember  Enable autologin
	 *
	 * @return  \App\Entities\Auth\UserEntity | \CodeIgniter\HTTP\RedirectResponse
	 */
	public function login($email, $password, bool $remember = false)
	{
		$options		= config('Auth')->options;
		$imap_host		= config('Auth')->imap_host;
		$imap_port		= config('Auth')->imap_port;
		$imap_domain	= config('Auth')->imap_domain;
		$imap_flags		= config('Auth')->imap_flags;

		list($username, ) = explode('@', $email, 2);

        $host = '{'.$imap_host.':'.$imap_port.$imap_flags.'}';
        $mbox = imap_open($host, $username.'@'.$imap_domain, $password);
		if( ! $mbox)
		{
			return redirect()
				->to('/auth/login')
				->with('errors', lang('Auth.wrong_user'));
		}

		$user = new User();
		$userEntity = $user->where('email', $email)->first();
		if (! $userEntity instanceof UserEntity)
		{
			$UserEntity = new UserEntity();
			$UserEntity->name = $username;
			$UserEntity->email = $email;
			$UserEntity->password = password_hash('12345678', PASSWORD_DEFAULT);
			$UserEntity->mobile = '0';
			$UserEntity->options = $options;
			$UserEntity->authcode = '';
			$UserEntity->status = 'On';
			$UserEntity->created_by = '0';
			$UserEntity->created_at = date('Y-m-d H:i:s');
			$user->save($UserEntity);
			// Get user row
			$userEntity = $user->where('email', $email)->first();
		}

		if (! $userEntity instanceof UserEntity)
		{
			return redirect()
				->to('/auth/login')
				->with('errors', lang('Auth.wrong_user'));
		}

		return $userEntity;
	}

	//--------------------------------------------------------------------

	/**
	 * User Register.
	 *
	 * @param   string   $email     user email
	 * @param   string   $name      user name
	 * @param   string   $mobile    user mobile
	 * @param   string   $password  user password
	 *
	 * @return  boolean
	 */
	public function register($email, $name, $mobile, $password)
	{

	}

	//--------------------------------------------------------------------

	/**
	 * User Lost.
	 *
	 * @param   string   $email     user email
	 *
	 * @return  boolean
	 */
	public function lost($email)
	{

	}

	//--------------------------------------------------------------------

	/**
	 * User Reset.
	 *
	 * @param   string   $email     user email
	 * @param   string   $code      user code
	 * @param   string   $password  user password
	 *
	 * @return  boolean
	 */
	public function reset($email, $code, $password)
	{

	}

	//--------------------------------------------------------------------

	/**
	 * User logout.
	 *
	 * @param   string   $email     user email
	 *
	 * @return  boolean
	 */
	public function logout($email)
	{

	}

	//--------------------------------------------------------------------
}
