<?php namespace App\Controllers\Auth\Drivers;

use App\Models\Auth\User;
use App\Entities\Auth\UserEntity;

/**
 * Authenticate users using LDAP
 */

class LdapDriver implements AuthInterface {

	/**
	 * User login.
	 *
	 * @param   string   $email     user email
	 * @param   string   $password  user password
	 * @param   boolean  $remember  Enable autologin
	 *
	 * @return  \App\Entities\Auth\UserEntity | \CodeIgniter\HTTP\RedirectResponse
	 */
	public function login($email, $password, bool $remember = false)
	{
		$options				= config('Auth')->options;
		$ldap_host				= config('Auth')->ldap_host;
		$ldap_port				= config('Auth')->ldap_port;
		$ldap_domain			= config('Auth')->ldap_domain;
		$ldap_allowed_ous       = config('Auth')->ldap_allowed_ous;
		$ldap_base_dn           = config('Auth')->ldap_base_dn;
		$ldap_sam_account_name  = config('Auth')->ldap_sam_account_name;

		// user is invalid by default
		$user_is_valid			= false;

		//remove LDAP special characters and domain
		$username = str_replace(['*','(', ')', '\\', 'NUL', '/','@'.$ldap_domain], '', $email);
		if(strpos($username, '@'))
		{
			list($username, ) = explode('@', $username, 2);
		}
		$conn = ldap_connect($ldap_host, $ldap_port);
		// always set protocol version to 3 
		@ldap_set_option($conn, LDAP_OPT_PROTOCOL_VERSION, 3) ;
		$bind = ldap_bind($conn, $username.'@'.$ldap_domain, $password);
		if($bind)
		{
			$filter		= "(|($ldap_sam_account_name=$username)(mail=$username@*))";
			$result		= ldap_search($conn, $ldap_base_dn, $filter, array('memberof','distinguishedname'));
			$entries 	= ldap_get_entries($conn, $result);
			if($entries['count']>0)
			{
				// check if user in allowed OUs using memberof
				if( ! empty($ldap_allowed_ous) && ! empty($entries['0']['memberof']))
				{
					foreach ($entries['0']['memberof'] as $key => $memberof)
					{
						if(preg_match('/'.preg_quote($ldap_allowed_ous).'/i', $memberof))
						{
							$user_is_valid = true;
						}
					}
				}
				// check if user in allowed OUs using memberof
				if( ! empty($ldap_allowed_ous) && ! empty($entries['0']['distinguishedname']))
				{
					if(preg_match('/'.preg_quote($ldap_allowed_ous).'/i', $entries['0']['distinguishedname']))
					{
						$user_is_valid = true;
					}
				}
			}
		}

		if($user_is_valid !== true)
		{
			return redirect()
				->to('/auth/login')
				->with('errors', lang('Auth.wrong_user'));
		}

		$user = new User();
		$userEntity = $user->where('email', $email)->first();
		if (! $userEntity instanceof UserEntity)
		{
			$UserEntity = new UserEntity();
			$UserEntity->name = $username;
			$UserEntity->email = $email;
			$UserEntity->password = password_hash('12345678', PASSWORD_DEFAULT);
			$UserEntity->mobile = '0';
			$UserEntity->options = $options;
			$UserEntity->authcode = '';
			$UserEntity->status = 'On';
			$UserEntity->created_by = '0';
			$UserEntity->created_at = date('Y-m-d H:i:s');
			$user->save($UserEntity);
			// Get user row
			$userEntity = $user->where('email', $email)->first();
		}

		if (! $userEntity instanceof UserEntity)
		{
			return redirect()
				->to('/auth/login')
				->with('errors', lang('Auth.wrong_user'));
		}

		return $userEntity;
	}

	//--------------------------------------------------------------------

	/**
	 * User Register.
	 *
	 * @param   string   $email     user email
	 * @param   string   $name      user name
	 * @param   string   $mobile    user mobile
	 * @param   string   $password  user password
	 *
	 * @return  boolean
	 */
	public function register($email, $name, $mobile, $password)
	{

	}

	//--------------------------------------------------------------------

	/**
	 * User Lost.
	 *
	 * @param   string   $email     user email
	 *
	 * @return  boolean
	 */
	public function lost($email)
	{

	}

	//--------------------------------------------------------------------

	/**
	 * User Reset.
	 *
	 * @param   string   $email     user email
	 * @param   string   $code      user code
	 * @param   string   $password  user password
	 *
	 * @return  boolean
	 */
	public function reset($email, $code, $password)
	{

	}

	//--------------------------------------------------------------------

	/**
	 * User logout.
	 *
	 * @param   string   $email     user email
	 *
	 * @return  boolean
	 */
	public function logout($email)
	{

	}

	//--------------------------------------------------------------------
}
