<?php namespace App\Controllers\Auth;

use App\Models\Auth\User;
use App\Entities\Auth\UserEntity;
use App\Controllers\BaseController;

class Login extends BaseController
{
	protected $user;
	function __construct()
	{
		$this->user = new User();
	}

	public function index()
	{
		return view('auth/login/index');
	}

	public function postIndex()
	{
		$rules = [
			'email'     => 'required|valid_email',
			'password'  => 'required|min_length[8]',
		];
		if( ! $this->validate($rules))
		{
			$errors = $this->validator->getErrors();
			$errors = implode('<br />', $errors);
			return redirect()->to('/auth/login')->withInput()->with('errors', $errors);
		}

		$email      = $this->request->getVar('email', FILTER_SANITIZE_EMAIL);
		$password   = $this->request->getVar('password', FILTER_SANITIZE_STRING);
		// check user using database
		$auth 		= config('Auth');
		$authDriver = new $auth->driver;
		
		$return		= $authDriver->login($email, $password, false);
		if (! $return instanceof UserEntity)
		{
			return $return;
		}

		if ($return->status != 'On')
		{
		   return redirect()
				->to('/auth/login')
				->with('errors', lang('Auth.disabled'));
		}

		session()->regenerate();
		session()->set('user', $return);
		session()->set('user_id', $return->id);

		return redirect()
			->to('/');
	}

	public function logout()
	{
	   session()->remove('user');
	   session()->remove('user_id');
	   session()->regenerate(true);
	   return redirect()
			->to('/auth/login');
	}

}
