<?php namespace App\Models;

use App\Models\Model;
use App\Entities\LookupEntity;

class Lookup extends Model
{
    protected $table = 'lookups';
    protected $returnType = LookupEntity::class;
    protected $useTimestamps = false;
    protected $allowedFields = [
        'name','value','rules','sort'
    ];

    protected $validationRules = [
        'name'      => 'required|min_length[1]',
        'value'     => 'required',
        'rules'     => '',
        'sort'      => 'is_natural',
    ];
    protected $validationMessages = [
        'name' => [
            'required'  => NULL,
        ],
        'value' => [
            'required'  => NULL,
        ],
        'rules' => [
            'required'  => NULL,
        ],
        'sort' => [
            'required'  => NULL,
        ],
        'status' => [
            'required'  => NULL,
        ],
    ];

    /**
        get by name
     */
    public function findByName($name, $key = null)
    {
        $builder = $this->builder();
        $builder->like('name', $name, 'after');
        if($key !== null)
        {
            $key = is_array($key)?$key:[$key];
            $row = $builder->whereIn($this->table . '.value', $key)->get();
        }
        else
        {
            $row  = $builder->get();
        }
        return $row->getResult($this->returnType);
    }

    /**
        get status lookups
     */
    public function status($key = null)
    {
        return $this->findByName('status', $key);
    }
}
